function NeedsSciencePack(tech, pack)
	-- See if it needs specified science pack.
	for _, ingredient in pairs(tech.unit.ingredients) do
		if ingredient[1] == pack then
			return true
		end
	end
	
	return false
end

function HasPrereq(tech, pack, pretech)
	-- Check if one of the prerequisites needs specified pack or has specified research as its prerequisite.
	if tech.prerequisites then
		for _, prereqname in pairs(tech.prerequisites) do
			if prereqname == pretech then
				return true
			end
			local prereq = data.raw.technology[prereqname]
			if NeedsSciencePack(prereq, pack) or HasPrereq(prereq, pack, pretech) then
				return true
			end
		end
	end
	return false
end


function ShouldNeedBattery(tech)
	-- If it (or any of its prerequisites) has battery as its prerequisite.
	-- Or any of its prerequisites needs science 3, then this one doesn't need the prerequisite.
	if HasPrereq(tech, "science-pack-3", "battery") then
		return false
	end
	if NeedsSciencePack(tech, "science-pack-3") then
		return true
	end
	return false
end

function ShouldNeedAdvElec(tech)
	-- If it (or any of its prerequisites) has advanced-electronics as its prerequisite.
	-- Or any of its prerequisites needs science 3, then this one doesn't need the prerequisite.
	if HasPrereq(tech, "science-pack-3", "advanced-electronics") then
		return false
	end
	if NeedsSciencePack(tech, "science-pack-3") then
		return true
	end
	return false
end

function ShouldNeedAlienTech(tech)
	-- If it (or any of its prerequisites) has alien-technology as its prerequisite.
	-- Or any of its prerequisites needs science 4, then this one doesn't need the prerequisite.
	if HasPrereq(tech, "alien-science-pack", "alien-technology") then
		return false
	end
	if NeedsSciencePack(tech, "alien-science-pack") then
		return true
	end
	return false
end

-- This code is actually executed.

for _, tech in pairs(data.raw.technology) do
	if ShouldNeedBattery(tech) then
		table.insert(tech.prerequisites, "battery")
	end
	if ShouldNeedAdvElec(tech) then
		table.insert(tech.prerequisites, "advanced-electronics")
	end
	if ShouldNeedAlienTech(tech) then
		table.insert(tech.prerequisites, "alien-technology")
	end
end